@@ -19,7 +19,8 @@
 
 #include "triggers/triggers.h"
 #include "action_data/action_data.h"
-#include "windows.h"
+#include "windows_handler.h"
+#include "windows_helper/window_selection_list.h"
 
 #include <KDE/KConfigGroup>
 #include <KDE/KDebug>
@@ -28,22 +29,36 @@
 
 namespace KHotKeys {
 
-WindowTrigger::WindowTrigger( ActionData* data_P, Windowdef_list* windows_P,
-     int window_actions_P )
-    : Trigger( data_P ), _windows( windows_P ), window_actions( window_actions_P ),
-      last_active_window( None ), active( false )
+WindowTrigger::WindowTrigger(
+        ActionData* data_P,
+        Windowdef_list* windows_P,
+        WindowEvents window_actions_P)
+    :   Trigger( data_P ),
+        _windows( windows_P ),
+        window_actions( window_actions_P ),
+        existing_windows(),
+        last_active_window( None ),
+        active( true )
     {
+    if (!_windows)
+        {
+        _windows = new Windowdef_list( "Windowdef_list comment");
+        }
+
+    Q_ASSERT(_windows->isEmpty());
     init();
     }
 
 
-WindowTrigger::WindowTrigger( KConfigGroup& cfg_P, ActionData* data_P )
-    : Trigger( cfg_P, data_P ), active( false )
+WindowTrigger::WindowTrigger(
+        KConfigGroup& cfg_P,
+        ActionData* data_P)
+    :   Trigger( cfg_P, data_P ),
+        active( true )
     {
-//    kDebug() << "WindowTrigger";
     KConfigGroup windowsConfig( cfg_P.config(), cfg_P.name() + "Windows" );
     _windows = new Windowdef_list( windowsConfig );
-    window_actions = cfg_P.readEntry( "WindowActions",0 );
+    window_actions = WindowEvents(cfg_P.readEntry( "WindowActions", 0 ));
     init();
     }
 
@@ -55,6 +70,7 @@ WindowTrigger::~WindowTrigger()
     delete _windows;
     }
 
+
 void WindowTrigger::init()
     {
     kDebug() << "WindowTrigger::init()";
@@ -70,30 +86,43 @@ void WindowTrigger::init()
 
 void WindowTrigger::activate( bool activate_P )
     {
-    active = activate_P && khotkeys_active();
+    active = activate_P;
     }
 
 
 void WindowTrigger::active_window_changed( WId window_P )
     {
-    bool was_match = false;
-    if( existing_windows.contains( last_active_window ))
-        was_match = existing_windows[ last_active_window ];
-    if( active && was_match && ( window_actions & WINDOW_DEACTIVATES ))
+    if (!existing_windows.contains(window_P))
+        {
+        existing_windows[window_P] = windows()->match( Window_data( window_P ));
+        }
+
+    if (!active || !khotkeys_active())
+        {
+        // We still keep track of the last active window so we have valid data
+        // if khotkeys is switched on again.
+        last_active_window = window_P;
+        return;
+        }
+
+    // Check if the last active window was a match for us
+    bool was_match = existing_windows.contains(last_active_window)
+        ? existing_windows[last_active_window]
+        : false;
+
+    if (was_match && (window_actions & WINDOW_DEACTIVATES))
         {
         windows_handler->set_action_window( window_P );
         data->execute();
         }
-/*    bool matches = windows()->match( Window_data( window_P ));
-    existing_windows[ window_P ] = matches;*/
-    bool matches = existing_windows.contains( window_P )
-        ? existing_windows[ window_P ] : false;
-    if( active && matches && ( window_actions & WINDOW_ACTIVATES ))
+
+    if (existing_windows[window_P] && ( window_actions & WINDOW_ACTIVATES))
         {
+        kDebug() << "Executing data";
         windows_handler->set_action_window( window_P );
         data->execute();
         }
-    kDebug() << "WindowTrigger::a_w_changed() : " << was_match << "|" << matches;
+
     last_active_window = window_P;
     }
 
@@ -102,8 +131,11 @@ void WindowTrigger::cfg_write( KConfigGroup& cfg_P ) const
     {
     base::cfg_write( cfg_P );
     KConfigGroup windowsConfig( cfg_P.config(), cfg_P.name() + "Windows" );
-    windows()->cfg_write( windowsConfig );
-    cfg_P.writeEntry( "WindowActions", window_actions );
+    if (windows())
+        {
+        windows()->cfg_write( windowsConfig );
+        }
+    cfg_P.writeEntry( "WindowActions", static_cast<int>(window_actions));
     cfg_P.writeEntry( "Type", "WINDOW" ); // overwrites value set in base::cfg_write()
     }
 
@@ -114,6 +146,12 @@ const QString WindowTrigger::description() const
     }
 
 
+void WindowTrigger::setOnWindowEvents(WindowEvents events)
+    {
+    window_actions = events;
+    }
+
+
 bool WindowTrigger::triggers_on( window_action_t w_action_P ) const
     {
     return window_actions & w_action_P;
@@ -122,12 +160,17 @@ bool WindowTrigger::triggers_on( window_action_t w_action_P ) const
 
 void WindowTrigger::window_added( WId window_P )
     {
-    bool matches = windows()->match( Window_data( window_P ));
-    existing_windows[ window_P ] = matches;
-    kDebug() << "WindowTrigger::w_added() : " << matches;
-    if( active && matches && ( window_actions & WINDOW_APPEARS ))
+    // Always keep track of windows,
+    existing_windows[window_P] = windows()->match( Window_data( window_P ));
+
+    if (!active || !khotkeys_active())
         {
-        windows_handler->set_action_window( window_P );
+        return;
+        }
+
+    if (existing_windows[window_P] && (window_actions & WINDOW_APPEARS))
+        {
+        windows_handler->set_action_window(window_P);
         data->execute();
         }
     }
@@ -135,49 +178,59 @@ void WindowTrigger::window_added( WId window_P )
 
 void WindowTrigger::window_removed( WId window_P )
     {
-    if( existing_windows.contains( window_P ))
+    // Always keep track of windows,
+    bool matches = false;
+    if (existing_windows.contains(window_P))
         {
-        bool matches = existing_windows[ window_P ];
-        kDebug() << "WindowTrigger::w_removed() : " << matches;
-        if( active && matches && ( window_actions & WINDOW_DISAPPEARS ))
-            {
-            windows_handler->set_action_window( window_P );
-            data->execute();
-            }
+        matches = existing_windows[window_P];
         existing_windows.remove( window_P );
-        // CHECKME jenze co kdyz se window_removed zavola pred active_window_changed ?
         }
-    else
-        kDebug() << "WindowTrigger::w_removed()";
+
+    if (!active || !khotkeys_active())
+        {
+        return;
+        }
+
+    if (matches && (window_actions & WINDOW_DISAPPEARS))
+        {
+        windows_handler->set_action_window( window_P );
+        data->execute();
+        }
     }
 
 
 void WindowTrigger::window_changed( WId window_P, unsigned int dirty_P )
-    { // CHECKME snad nebude mit vliv, kdyz budu kaslat na properties_P a zkratka
-      // kontrolovat kazdou zmenu
-      // CHECKME kdyz se zmeni okno z match na non-match, asi to nebrat jako DISAPPEAR
-    if( ! ( dirty_P & ( NET::WMName | NET::WMWindowType )))
+    {
+    if (! (dirty_P & (NET::WMName | NET::WMWindowType)))
         return;
-    kDebug() << "WindowTrigger::w_changed()";
+
+    // Check if the old state was a match
     bool was_match = false;
-    if( existing_windows.contains( window_P ))
-        was_match = existing_windows[ window_P ];
+    if (existing_windows.contains(window_P))
+        was_match = existing_windows[window_P];
+
+    // Check if the new state is a match
     bool matches = windows()->match( Window_data( window_P ));
-    existing_windows[ window_P ] = matches;
-    if( active && matches && !was_match )
+    existing_windows[window_P] = matches;
+
+    if (!active || !khotkeys_active())
         {
-        if( window_actions & WINDOW_APPEARS )
+        return;
+        }
+
+    if (matches && !was_match)
+        {
+        if (window_actions & WINDOW_APPEARS)
             {
             windows_handler->set_action_window( window_P );
             data->execute();
             }
-        else if( window_actions & WINDOW_ACTIVATES && window_P == windows_handler->active_window())
+        else if (window_actions & WINDOW_ACTIVATES && window_P == windows_handler->active_window())
             {
             windows_handler->set_action_window( window_P );
             data->execute();
             }
         }
-    kDebug() << "WindowTrigger::w_changed() : " << was_match << "|" << matches;
     }
 
 
@@ -187,6 +240,12 @@ const Windowdef_list* WindowTrigger::windows() const
     }
 
 
+Windowdef_list* WindowTrigger::windows()
+    {
+    return _windows;
+    }
+
+
 WindowTrigger* WindowTrigger::copy( ActionData* data_P ) const
     {
     WindowTrigger* ret = new WindowTrigger( data_P ? data_P : data, windows()->copy(),
