@@ -276,6 +276,11 @@ void Klipper::slotStartShowTimer()
 
 void Klipper::loadSettings()
 {
+    // Security bug 142882: If user has save clipboard turned off, old data should be deleted from disk
+    if (m_bKeepContents && !KlipperSettings::keepClipboardContents()) {
+        saveHistory(true);
+    }
+
     m_bPopupAtMouse = KlipperSettings::popupAtMousePosition();
     m_bKeepContents = KlipperSettings::keepClipboardContents();
     m_bReplayActionInHistory = KlipperSettings::replayActionInHistory();
@@ -410,7 +415,7 @@ bool Klipper::loadHistory() {
     return true;
 }
 
-void Klipper::saveHistory() {
+void Klipper::saveHistory(bool empty) {
     static const char* const failed_save_warning =
         "Failed to save history. Clipboard history cannot be saved.";
     // don't use "appdata", klipper is also a kicker applet
@@ -428,10 +433,12 @@ void Klipper::saveHistory() {
     QDataStream history_stream( &data, QIODevice::WriteOnly );
     history_stream << klipper_version; // const char*
 
-    History::iterator it = history()->youngest();
-    while (it.hasNext()) {
-        const HistoryItem *item = it.next();
-        history_stream << item;
+    if (!empty) {
+        History::iterator it = history()->youngest();
+        while (it.hasNext()) {
+            const HistoryItem *item = it.next();
+            history_stream << item;
+        }
     }
 
     quint32 crc = crc32( 0, reinterpret_cast<unsigned char *>( data.data() ), data.size() );
@@ -1029,5 +1036,4 @@ static void ensureGlobalSyncOff(KSharedConfigPtr config) {
 
 }
 
-
 #include "klipper.moc"
