@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2007-2008 Ryan P. Bitanga <ryan.bitanga@gmail.com>
+ *   Copyright (C) 2007-2009 Ryan P. Bitanga <ryan.bitanga@gmail.com>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -43,6 +43,7 @@
 #include "qs_statusbar.h"
 #include "qs_matchitem.h"
 #include "qs_matchview.h"
+#include "qs_querymatchitem.h"
 
 //Widget dimensions
 const int WIDTH = 390;
@@ -74,6 +75,7 @@ class QsMatchView::Private
         bool m_hasFocus;
         bool m_itemsRemoved;
         bool m_listVisible;
+        bool m_selectionMade;
 };
 
 QsMatchView::QsMatchView(QWidget *parent)
@@ -85,6 +87,7 @@ QsMatchView::QsMatchView(QWidget *parent)
     d->m_hasFocus = false;
     d->m_itemsRemoved = false;
     d->m_listVisible = true;
+    d->m_selectionMade = false; //Prevent completion box from popping up once a user chooses a match
     //FIXME: don't hardcode black
     setStyleSheet("QListWidget {color: black} QLineEdit {color: black}");
 
@@ -157,8 +160,10 @@ void QsMatchView::reset()
 {
     clear(true);
 
+    d->m_stack->setCurrentIndex(0);
     d->m_arrowButton->hide();
     d->m_listVisible = true;
+    d->m_selectionMade = false;
     d->m_hasFocus = false;
     d->m_searchTerm = QString();
     d->m_compBox->clear();
@@ -172,26 +177,44 @@ void QsMatchView::reset()
     setDescriptionText(i18n("Type to search."));
 }
 
-void QsMatchView::setItems(const QList<MatchItem*> &items, bool popup)
+bool queryMatchCompare(MatchItem *a, MatchItem *b)
 {
+    QueryMatchItem *m1 = qobject_cast<QueryMatchItem*>(a);
+    QueryMatchItem *m2 = qobject_cast<QueryMatchItem*>(b);
+    if (m1 && m2) {
+        return m2->match() < m1->match();
+    }
+    return a < b;
+}
 
-    clear(true);
-
-    d->m_compBox->clear();
-
-    d->m_currentItem = -1;
-    d->m_items = items;
-
+void QsMatchView::setItems(const QList<MatchItem*> &items, bool popup, bool append)
+{
     int spacing = MatchItem::ITEM_SIZE/2;
 
     int pos = spacing;
 
+    d->m_currentItem = -1;
+
+    if (!append) {
+        clear(true);
+        d->m_compBox->clear();
+        d->m_items = items;
+    } else {
+        /*if (!d->m_items.isEmpty()) {
+            pos += d->m_items.last()->pos().x();
+        }*/
+        d->m_items << items;
+    }
+    qStableSort(d->m_items.begin(), d->m_items.end(), queryMatchCompare);
+
     foreach(MatchItem *item, d->m_items) {
         if (item) {
             item->setPos(pos, SMALL_ICON_PADDING);
             item->scale(0.5, 0.5);
             pos += spacing;
-            d->m_scene->addItem(item);
+            if (d->m_listVisible) {
+                d->m_scene->addItem(item);
+            }
             QString description;
             if (item->description().isEmpty()) {
                 description = item->name();
@@ -204,7 +227,14 @@ void QsMatchView::setItems(const QList<MatchItem*> &items, bool popup)
     }
     d->m_itemsRemoved = false;
     setItemCount(d->m_items.size());
+
+    if (d->m_selectionMade) {
+        //kDebug() << "A user selection was already made" << endl;
+        return;
+    }
+
     scrollToItem(0);
+
     //Ensure popup is shown if desired
     if (popup) {
         if (items.size()) {
@@ -262,7 +292,11 @@ void QsMatchView::setDescriptionText(const QString &text, const QColor &color)
     d->m_descRect->setBrush(b);
     d->m_descRect->setPen(p);
 
-    d->m_descText = new QGraphicsTextItem(text, d->m_descRect);
+    QFontMetrics fm(font());
+
+    // Show ellipsis in the middle to distinguish between strings with identical
+    // beginnings e.g. paths
+    d->m_descText = new QGraphicsTextItem(fm.elidedText(text, Qt::ElideMiddle, WIDTH), d->m_descRect);
     //Center text
     d->m_descText->setPos(-(d->m_descText->boundingRect().width()/2), 60);
 
@@ -306,11 +340,6 @@ void QsMatchView::toggleView()
 void QsMatchView::showLoading()
 {
     clear(true);
-//     QPointF pos[8];
-//     for (int i = 0; i < 8; ++i) {
-//         pos[i] = QPointF(std::sin((i * 6.28) / 8.0) * 25,
-//                          std::cos((i * 6.28) / 8.0) * 25);
-//     }
 
     d->m_descText = new QGraphicsTextItem(i18n("Loading..."), d->m_descRect);
     d->m_descText->setDefaultTextColor(QColor(Qt::white));
@@ -319,14 +348,6 @@ void QsMatchView::showLoading()
     //Center text
     d->m_descText->setPos(-(d->m_descText->boundingRect().width()/2), (HEIGHT - fm.height())/2);
     d->m_scene->addItem(d->m_descText);
-
-//     for (int i = 0; i < 8; ++i) {
-//         QGraphicsEllipseItem *item = new QGraphicsEllipseItem(0, 30, 6, 6);
-//         item->setPos(pos[i]);
-//         QBrush b(QColor(0, 0, 0, 255*i/8));
-//         item->setBrush(b);
-//         d->m_scene->addItem(item);
-//     }
 }
 
 void QsMatchView::showList()
@@ -352,7 +373,9 @@ void QsMatchView::showList()
 void QsMatchView::showSelected()
 {
     if (!d->m_items.size()) {
-        reset();
+        if (d->m_searchTerm.isEmpty()) {
+            reset();
+        }
         return;
     }
 
@@ -573,6 +596,7 @@ void QsMatchView::keyPressEvent(QKeyEvent *e)
         QWidget::keyPressEvent(e);
         return;
     }
+
     switch (e->key()) {
         case Qt::Key_Period:
             //Switch to line edit
@@ -606,6 +630,7 @@ void QsMatchView::keyPressEvent(QKeyEvent *e)
                        &&  d->m_currentItem < d->m_items.size()) {
                     emit itemActivated(d->m_items[d->m_currentItem]);
             }
+            d->m_selectionMade = true;
             showSelected();
             return;
         default:
@@ -613,16 +638,22 @@ void QsMatchView::keyPressEvent(QKeyEvent *e)
     }
 
     //Don't add control characters to the search term
-    foreach (QChar c, e->text()) {
+    foreach (const QChar &c, e->text()) {
         if (c.isPrint()) {
             if (d->m_stack->currentIndex() == 1) {
                 d->m_searchTerm = d->m_lineEdit->text() + c;
             } else {
                 d->m_searchTerm += c;
             }
+            d->m_selectionMade = false;
         }
     }
-    d->m_lineEdit->setText(d->m_searchTerm);
+    // If line edit has focus, not all keypresses make it to
+    // the parent widget, hence the internal search term is
+    // out of date
+    if (!d->m_lineEdit->hasFocus()) {
+        d->m_lineEdit->setText(d->m_searchTerm);
+    }
     QWidget::keyPressEvent(e);
 }
 
