@@ -72,6 +72,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "client.h"
 #include "shutdowndlg.h"
 
+#include <solid/powermanagement.h>
 
 #include <kdebug.h>
 
@@ -82,7 +83,9 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 void KSMServer::logout( int confirm, int sdtype, int sdmode )
 {
-    emit loggingOut( confirm, sdtype, sdmode );
+    // KDE5: remove me
+    if (sdtype == KWorkSpace::ShutdownTypeLogout)
+        sdtype = KWorkSpace::ShutdownTypeNone;
 
     shutdown( (KWorkSpace::ShutdownConfirm)confirm,
             (KWorkSpace::ShutdownType)sdtype,
@@ -128,6 +131,7 @@ void KSMServer::shutdown( KWorkSpace::ShutdownConfirm confirm,
         (confirm == KWorkSpace::ShutdownConfirmYes) ? false :
     (confirm == KWorkSpace::ShutdownConfirmNo) ? true :
                 !cg.readEntry( "confirmLogout", true );
+    bool choose = false;
     bool maysd = false;
     if (cg.readEntry( "offerShutdown", true ) && KDisplayManager().canShutdown())
         maysd = true;
@@ -137,9 +141,11 @@ void KSMServer::shutdown( KWorkSpace::ShutdownConfirm confirm,
             logoutConfirmed)
             return; /* unsupported fast shutdown */
         sdtype = KWorkSpace::ShutdownTypeNone;
-    } else if (sdtype == KWorkSpace::ShutdownTypeDefault)
+    } else if (sdtype == KWorkSpace::ShutdownTypeDefault) {
         sdtype = (KWorkSpace::ShutdownType)
                 cg.readEntry( "shutdownType", (int)KWorkSpace::ShutdownTypeNone );
+        choose = true;
+    }
     if (sdmode == KWorkSpace::ShutdownModeDefault)
         sdmode = KWorkSpace::ShutdownModeInteractive;
 
@@ -148,7 +154,7 @@ void KSMServer::shutdown( KWorkSpace::ShutdownConfirm confirm,
     if ( !logoutConfirmed ) {
         KSMShutdownFeedback::start(); // make the screen gray
         logoutConfirmed =
-            KSMShutdownDlg::confirmShutdown( maysd, sdtype, bopt );
+            KSMShutdownDlg::confirmShutdown( maysd, choose, sdtype, bopt );
         // ###### We can't make the screen remain gray while talking to the apps,
         // because this prevents interaction ("do you want to save", etc.)
         // TODO: turn the feedback widget into a list of apps to be closed,
@@ -158,6 +164,10 @@ void KSMServer::shutdown( KWorkSpace::ShutdownConfirm confirm,
 
     if ( logoutConfirmed ) {
 
+        // If the logout was confirmed, let's start a powermanagement inhibition.
+        // We store the cookie so we can interrupt it if the logout will be canceled
+        inhibitCookie = Solid::PowerManagement::beginSuppressingSleep();
+
         shutdownType = sdtype;
         shutdownMode = sdmode;
         bootOption = bopt;
@@ -357,6 +367,7 @@ void KSMServer::handlePendingInteractions()
 
 void KSMServer::cancelShutdown( KSMClient* c )
 {
+    Solid::PowerManagement::stopSuppressingSleep(inhibitCookie);
     kDebug( 1218 ) << "Client " << c->program() << " (" << c->clientId() << ") canceled shutdown.";
     KNotification::event( "cancellogout" , i18n( "Logout canceled by '%1'", c->program()),
         QPixmap() , 0l , KNotification::DefaultEvent  );
