@@ -38,6 +38,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "strategies/desktopsortingstrategy.h"
 #include "strategies/programgroupingstrategy.h"
 #include "strategies/manualgroupingstrategy.h"
+#include "strategies/kustodiangroupingstrategy.h"
 #include "strategies/manualsortingstrategy.h"
 
 namespace TaskManager
@@ -60,23 +61,28 @@ public:
           showOnlyCurrentScreen(false),
           showOnlyMinimized(false),
           onlyGroupWhenFull(false),
-          changingGroupingStragegy(false)
+          changingGroupingStrategy(false)
     {
     }
 
     /** reload all tasks from TaskManager */
     void reloadTasks();
+    void actuallyReloadTasks();
 
     /**
     * Keep track of changes in Taskmanager
     */
     void currentDesktopChanged(int);
     void taskChanged(TaskPtr, ::TaskManager::TaskChanges);
-
     void checkScreenChange();
-
-    void itemDestroyed();
+    void taskItemDestroyed(AbstractGroupableItem *);
+    void startupItemDestroyed(AbstractGroupableItem *);
     void checkIfFull();
+    void actuallyCheckIfFull();
+    bool addTask(TaskPtr);
+    void removeTask(TaskPtr);
+    void addStartup(StartupPtr);
+    void removeStartup(StartupPtr);
 
     GroupManager *q;
     QHash<TaskPtr, TaskItem*> itemList; //holds all tasks of the Taskmanager
@@ -89,13 +95,15 @@ public:
     AbstractSortingStrategy *abstractSortingStrategy;
     int currentScreen;
     QTimer screenTimer;
-    QList<TaskPtr> geometryTasks;
+    QTimer reloadTimer;
+    QTimer checkIfFullTimer;
+    QSet<TaskPtr> geometryTasks;
     int groupIsFullLimit;
     bool showOnlyCurrentDesktop : 1;
     bool showOnlyCurrentScreen : 1;
     bool showOnlyMinimized : 1;
     bool onlyGroupWhenFull : 1;
-    bool changingGroupingStragegy : 1;
+    bool changingGroupingStrategy : 1;
     QUuid configToken;
 };
 
@@ -106,15 +114,25 @@ GroupManager::GroupManager(QObject *parent)
     : QObject(parent),
       d(new GroupManagerPrivate(this))
 {
-    connect(TaskManager::self(), SIGNAL(taskAdded(TaskPtr)), this, SLOT(add(TaskPtr)));
-    connect(TaskManager::self(), SIGNAL(taskRemoved(TaskPtr)), this, SLOT(remove(TaskPtr)));
-    connect(TaskManager::self(), SIGNAL(startupAdded(StartupPtr)), this, SLOT(add(StartupPtr)));
-    connect(TaskManager::self(), SIGNAL(startupRemoved(StartupPtr)), this, SLOT(remove(StartupPtr)));
+    connect(TaskManager::self(), SIGNAL(taskAdded(TaskPtr)), this, SLOT(addTask(TaskPtr)));
+    connect(TaskManager::self(), SIGNAL(taskRemoved(TaskPtr)), this, SLOT(removeTask(TaskPtr)));
+    connect(TaskManager::self(), SIGNAL(startupAdded(StartupPtr)), this, SLOT(addStartup(StartupPtr)));
+    connect(TaskManager::self(), SIGNAL(startupRemoved(StartupPtr)), this, SLOT(removeStartup(StartupPtr)));
+
     d->rootGroup = new TaskGroup(this, "RootGroup", Qt::transparent);
-    //reloadTasks();
+
+    d->reloadTimer.setSingleShot(true);
+    d->reloadTimer.setInterval(0);
+    connect(&d->reloadTimer, SIGNAL(timeout()), this, SLOT(actuallyReloadTasks()));
+
     d->screenTimer.setSingleShot(true);
     d->screenTimer.setInterval(100);
     connect(&d->screenTimer, SIGNAL(timeout()), this, SLOT(checkScreenChange()));
+
+    d->checkIfFullTimer.setSingleShot(true);
+    d->checkIfFullTimer.setInterval(0);
+    connect(&d->checkIfFullTimer, SIGNAL(timeout()), this, SLOT(actuallyCheckIfFull()));
+
 }
 
 GroupManager::~GroupManager()
@@ -128,51 +146,63 @@ GroupManager::~GroupManager()
 
 void GroupManagerPrivate::reloadTasks()
 {
-    //kDebug() << "number of tasks available " << TaskManager::self()->tasks().size();
+    reloadTimer.start();
+}
 
-    QList <TaskPtr> taskList = TaskManager::self()->tasks().values();
-    foreach (const TaskPtr& task, taskList) { //Add all existing tasks
-        if (!q->add(task)) {
-            q->remove(task); //remove what isn't needed anymore
+void GroupManagerPrivate::actuallyReloadTasks()
+{
+    //kDebug() << "number of tasks available " << TaskManager::self()->tasks().size();
+    QHash<WId, TaskPtr> taskList = TaskManager::self()->tasks();
+    QMutableHashIterator<WId, TaskPtr> it(taskList);
+
+
+    while (it.hasNext()) {
+        it.next();
+
+        if (addTask(it.value())) {
+            it.remove();
         }
-        taskList.removeAll(task);
     }
 
-    foreach (const TaskPtr& task, taskList) { //Remove the remaining
-        q->remove(task);
+    // Remove what remains
+    it.toFront();
+    while (it.hasNext()) {
+        it.next();
+        removeTask(it.value());
     }
 
     emit q->reload();
 }
 
-void GroupManager::add(StartupPtr task)
+void GroupManagerPrivate::addStartup(StartupPtr task)
 {
     //kDebug();
-    TaskItem *item;
-    if (!d->startupList.contains(task)) {
-        item = new TaskItem(this, task);
-        d->startupList.insert(task, item); 
-        d->rootGroup->add(item);
+    if (!startupList.contains(task)) {
+        TaskItem *item = new TaskItem(q, task);
+        startupList.insert(task, item); 
+        rootGroup->add(item);
+        QObject::connect(item, SIGNAL(destroyed(AbstractGroupableItem*)),
+                         q, SLOT(startupItemDestroyed(AbstractGroupableItem*)));
     }
 }
 
-void GroupManager::remove(StartupPtr task)
+void GroupManagerPrivate::removeStartup(StartupPtr task)
 {
     //kDebug() << "startup";
-    if (!d->startupList.contains(task)) {
+    if (!startupList.contains(task)) {
         kDebug() << "invalid startup task";
         return;
     }
 
-    AbstractItemPtr item = d->startupList.take(task);
+    AbstractGroupableItem *item = startupList.take(task);
     if (item->parentGroup()) {
         item->parentGroup()->remove(item);
     }
 
-    emit itemRemoved(item);
+    emit q->itemRemoved(item);
 }
 
-bool GroupManager::add(TaskPtr task)
+bool GroupManagerPrivate::addTask(TaskPtr task)
 {
     /* kDebug() << task->visibleName()
              << task->visibleNameWithState()
@@ -185,22 +215,22 @@ bool GroupManager::add(TaskPtr task)
         return false;
     }
 
+    if (showOnlyCurrentScreen && !task->isOnScreen(currentScreen)) {
+        //kDebug() << "Not on this screen and showOnlyCurrentScreen";
+        return false;
+    }
+
     // Should the Task be displayed ? We always display if attention is demaded
     if (!task->demandsAttention()) {
         // As the Task doesn't demand attention
         // go through all filters whether the task should be displayed or not
-        if (showOnlyCurrentDesktop() && !task->isOnCurrentDesktop()) {
+        if (showOnlyCurrentDesktop && !task->isOnCurrentDesktop()) {
             /* kDebug() << "Not on this desktop and showOnlyCurrentDesktop"
                      << KWindowSystem::currentDesktop() << task->desktop(); */
             return false;
         }
 
-        if (showOnlyCurrentScreen() && !task->isOnScreen(d->currentScreen)) {
-            //kDebug() << "Not on this screen and showOnlyCurrentScreen";
-            return false;
-        }
-
-        if (showOnlyMinimized() && !task->isMinimized()) {
+        if (showOnlyMinimized && !task->isMinimized()) {
             //kDebug() << "Not minimized and only showing minimized";
             return false;
         }
@@ -229,52 +259,57 @@ bool GroupManager::add(TaskPtr task)
 
     //Ok the Task should be displayed
     TaskItem *item = 0;
-    if (!d->itemList.contains(task)) {
-        //Lookout for existing startuptask of this task
-        QMutableHashIterator<StartupPtr, TaskItem*> it(d->startupList);
-        while (it.hasNext()) {
-            it.next();
+    if (itemList.contains(task)) {
+        // we pretend to add it again so the group strategy evaluates it again
+        item = itemList.value(task);
+    } else {
+        // first search for an existing startuptask for this task
+        QHash<StartupPtr, TaskItem *>::iterator it = startupList.begin();
+        QHash<StartupPtr, TaskItem *>::iterator itEnd = startupList.end();
+        while (it != itEnd) {
             if (it.key()->matchesWindow(task->window())) {
-                //kDebug() << "startup task";
+                //kDebug() << "startup task found";
                 item = it.value();
+                startupList.erase(it);
+                QObject::disconnect(item, 0, q, 0);
                 item->setTaskPointer(task);
-                it.remove();
                 break;
             }
+
+            ++it;
         }
 
         if (!item) {
-            item = new TaskItem(this, task);
+            item = new TaskItem(q, task);
         }
 
-        connect(item, SIGNAL(destroyed()), this, SLOT(itemDestroyed()));
-        d->itemList.insert(task, item); 
-    } else {
-        item = d->itemList.value(task); //we add it again so the group is evaluated again
+        QObject::connect(item, SIGNAL(destroyed(AbstractGroupableItem*)),
+                         q, SLOT(taskItemDestroyed(AbstractGroupableItem*)));
+        itemList.insert(task, item);
     }
 
+    geometryTasks.insert(task);
+
     //Find a fitting group for the task with GroupingStrategies
-    if (d->abstractGroupingStrategy && !task->demandsAttention()) { //do not group attetion tasks
-        d->abstractGroupingStrategy->handleItem(item);
+    if (abstractGroupingStrategy && !task->demandsAttention()) { //do not group attention tasks
+        abstractGroupingStrategy->handleItem(item);
     } else {
-        d->rootGroup->add(item);
+        rootGroup->add(item);
     }
 
     return true;
 }
 
 
-void GroupManager::remove(TaskPtr task)
+void GroupManagerPrivate::removeTask(TaskPtr task)
 {
     //kDebug() << "remove: " << task->visibleName();
-    if (!d->geometryTasks.isEmpty()) {
-        d->geometryTasks.removeAll(task);
-    }
+    geometryTasks.remove(task);
 
-    TaskItem *item = d->itemList.value(task);
+    TaskItem *item = itemList.value(task);
     if (!item) {
         // this can happen if the window hasn't been caught previously, 
-        // of it its an ignored type such as a NET::Utility type window
+        // of it it is an ignored type such as a NET::Utility type window
         //kDebug() << "invalid item";
         return;
     }
@@ -283,21 +318,26 @@ void GroupManager::remove(TaskPtr task)
         item->parentGroup()->remove(item);
     }
 
-    emit itemRemoved(item);
+    emit q->itemRemoved(item);
+
+    itemList.remove(task);
+    //FIXME: We keep it in the itemlist because it may return: what does actually mean?
     //the item must exist as long as the TaskPtr does because of activate calls so don't delete the item here, it will delete itself. We keep it in the itemlist because it may return
 }
 
-void GroupManagerPrivate::itemDestroyed()
+void GroupManagerPrivate::taskItemDestroyed(AbstractGroupableItem *item)
 {
-    TaskItem *taskItem = qobject_cast<TaskItem*>(q->sender());
-    TaskItem *item = itemList.take(itemList.key(taskItem));
-    if (!item) {
-        kDebug() << "invalid item";
-        return;
-    }
-    QObject::disconnect(item, 0, q, 0);
+    TaskItem *taskItem = static_cast<TaskItem*>(item);
+    itemList.remove(itemList.key(taskItem));
+    geometryTasks.remove(taskItem->task());
 }
 
+void GroupManagerPrivate::startupItemDestroyed(AbstractGroupableItem *item)
+{
+    TaskItem *taskItem = static_cast<TaskItem*>(item);
+    startupList.remove(startupList.key(taskItem));
+    geometryTasks.remove(taskItem->task());
+}
 
 bool GroupManager::manualGroupingRequest(AbstractGroupableItem* item, TaskGroup* groupItem)
 {
@@ -315,25 +355,28 @@ bool GroupManager::manualGroupingRequest(AbstractGroupableItem* item, TaskGroup*
 
 bool GroupManager::manualGroupingRequest(ItemList items)
 {
-    // kDebug();
-    if (d->abstractGroupingStrategy) {
-     //   kDebug() << d->abstractGroupingStrategy->type() << ManualGrouping;
-        if (d->abstractGroupingStrategy->type() == ManualGrouping) {
-      //      kDebug();
-            return (qobject_cast<ManualGroupingStrategy*>(d->abstractGroupingStrategy))->groupItems(items);
+    //kDebug();
+    if (d->abstractGroupingStrategy && d->abstractGroupingStrategy->type() == ManualGrouping) {
+        //kDebug();
+        ManualGroupingStrategy *strategy = qobject_cast<ManualGroupingStrategy*>(d->abstractGroupingStrategy);
+        if (strategy) {
+            return strategy->groupItems(items);
         }
     }
+
     return false;
 }
 
 bool GroupManager::manualSortingRequest(AbstractGroupableItem* taskItem, int newIndex)
 {
     //kDebug();
-    if (d->abstractSortingStrategy) {
-        if (d->abstractSortingStrategy->type() == ManualSorting) {
-            return (qobject_cast<ManualSortingStrategy*>(d->abstractSortingStrategy))->moveItem(taskItem, newIndex);
+    if (d->abstractSortingStrategy && d->abstractSortingStrategy->type() == ManualSorting) {
+        ManualSortingStrategy *strategy = qobject_cast<ManualSortingStrategy*>(d->abstractSortingStrategy);
+        if (strategy) {
+            return strategy->moveItem(taskItem, newIndex);
         }
     }
+
     return false;
 }
 
@@ -343,7 +386,6 @@ GroupPtr GroupManager::rootGroup() const
     return d->rootGroup;
 }
 
-
 void GroupManagerPrivate::currentDesktopChanged(int newDesktop)
 {
     //kDebug();
@@ -381,10 +423,8 @@ void GroupManagerPrivate::taskChanged(TaskPtr task, ::TaskManager::TaskChanges c
         show = task->isMinimized();
     }
 
-    if (changes & ::TaskManager::GeometryChanged) {
-        if (!geometryTasks.contains(task)) {
-            geometryTasks.append(task);
-        }
+    if (showOnlyCurrentScreen && changes & ::TaskManager::GeometryChanged) {
+        geometryTasks.insert(task);
 
         if (!screenTimer.isActive()) {
             screenTimer.start();
@@ -401,12 +441,14 @@ void GroupManagerPrivate::taskChanged(TaskPtr task, ::TaskManager::TaskChanges c
         return;
     }
 
+    show = show && (!showOnlyCurrentScreen || task->isOnScreen(currentScreen));
+
     if (show) {
         //kDebug() << "add(task);";
-        q->add(task);
+        addTask(task);
     } else {
         //kDebug() << "remove(task);";
-        q->remove(task);
+        removeTask(task);
     }
 }
 
@@ -414,6 +456,7 @@ void GroupManager::setScreen(int screen)
 {
     //kDebug() << "new Screen: " << screen;
     d->currentScreen = screen;
+    d->reloadTasks();
 }
 
 
@@ -422,13 +465,11 @@ void GroupManagerPrivate::checkScreenChange()
     //kDebug();
     foreach (const TaskPtr &task, geometryTasks) {
         if (task->isOnScreen(currentScreen)) {
-            q->add(task);
+            addTask(task);
         } else {
-            q->remove(task);
+            removeTask(task);
         }
     }
-
-    geometryTasks.clear();
 }
 
 
@@ -466,22 +507,22 @@ bool GroupManager::onlyGroupWhenFull() const
     return d->onlyGroupWhenFull;
 }
 
-void GroupManager::setOnlyGroupWhenFull(bool state)
+void GroupManager::setOnlyGroupWhenFull(bool onlyGroupWhenFull)
 {
-    //kDebug() << state;
-    if (d->onlyGroupWhenFull == state) {
+    //kDebug() << onlyGroupWhenFull;
+    if (d->onlyGroupWhenFull == onlyGroupWhenFull) {
         return;
     }
 
-    d->onlyGroupWhenFull = state;
+    d->onlyGroupWhenFull = onlyGroupWhenFull;
 
-    if (state) {
-        connect(d->rootGroup, SIGNAL(itemAdded(AbstractItemPtr)), this, SLOT(checkIfFull()));
-        connect(d->rootGroup, SIGNAL(itemRemoved(AbstractItemPtr)), this, SLOT(checkIfFull()));
+    disconnect(d->rootGroup, SIGNAL(itemAdded(AbstractGroupableItem *)), this, SLOT(checkIfFull()));
+    disconnect(d->rootGroup, SIGNAL(itemRemoved(AbstractGroupableItem *)), this, SLOT(checkIfFull()));
+
+    if (onlyGroupWhenFull) {
+        connect(d->rootGroup, SIGNAL(itemAdded(AbstractGroupableItem *)), this, SLOT(checkIfFull()));
+        connect(d->rootGroup, SIGNAL(itemRemoved(AbstractGroupableItem *)), this, SLOT(checkIfFull()));
         d->checkIfFull();
-    } else {
-        disconnect(d->rootGroup, SIGNAL(itemAdded(AbstractItemPtr)), this, SLOT(checkIfFull()));
-        disconnect(d->rootGroup, SIGNAL(itemRemoved(AbstractItemPtr)), this, SLOT(checkIfFull()));
     }
 }
 
@@ -495,17 +536,29 @@ void GroupManager::setFullLimit(int limit)
 }
 
 void GroupManagerPrivate::checkIfFull()
+{
+    // Start a timer so that if we have been triggered by a layouting
+    // we give time for it to finish up instead of starting a new one
+    // right away.
+    checkIfFullTimer.start();
+}
+
+void GroupManagerPrivate::actuallyCheckIfFull()
 {
     //kDebug();
-    if (!onlyGroupWhenFull || groupingStrategy != GroupManager::ProgramGrouping) {
+    if (!onlyGroupWhenFull ||
+        groupingStrategy != GroupManager::ProgramGrouping ||
+        changingGroupingStrategy) {
         return;
     }
 
     if (itemList.size() >= groupIsFullLimit) {
         if (!abstractGroupingStrategy) {
+            geometryTasks.clear();
             q->setGroupingStrategy(GroupManager::ProgramGrouping);
         }
     } else if (abstractGroupingStrategy) {
+        geometryTasks.clear();
         q->setGroupingStrategy(GroupManager::NoGrouping);
         //let the visualization thing we still use the programGrouping
         groupingStrategy = GroupManager::ProgramGrouping;
@@ -520,6 +573,7 @@ bool GroupManager::showOnlyCurrentScreen() const
 void GroupManager::setShowOnlyCurrentScreen(bool showOnlyCurrentScreen)
 {
     d->showOnlyCurrentScreen = showOnlyCurrentScreen;
+    d->reloadTasks();
 }
 
 bool GroupManager::showOnlyCurrentDesktop() const
@@ -530,6 +584,7 @@ bool GroupManager::showOnlyCurrentDesktop() const
 void GroupManager::setShowOnlyCurrentDesktop(bool showOnlyCurrentDesktop)
 {
     d->showOnlyCurrentDesktop = showOnlyCurrentDesktop;
+    d->reloadTasks();
 }
 
 bool GroupManager::showOnlyMinimized() const
@@ -557,17 +612,15 @@ void GroupManager::setSortingStrategy(TaskSortingStrategy sortOrder)
     //kDebug() << sortOrder;
 
     if (d->abstractSortingStrategy) {
-        if (d->abstractSortingStrategy->type() == sortOrder){
+        if (d->abstractSortingStrategy->type() == sortOrder) {
             return;
-        } else {
-            d->abstractSortingStrategy->deleteLater();
         }
+
+        d->abstractSortingStrategy->deleteLater();
+        d->abstractSortingStrategy = 0;
     }
 
     switch (sortOrder) {
-        case NoSorting: //manual and no grouping result both in non automatic grouping
-            d->abstractSortingStrategy = 0;
-            break;
         case ManualSorting:
             d->abstractSortingStrategy = new ManualSortingStrategy(this);
             d->abstractSortingStrategy->handleGroup(d->rootGroup);
@@ -583,9 +636,11 @@ void GroupManager::setSortingStrategy(TaskSortingStrategy sortOrder)
             d->abstractSortingStrategy->handleGroup(d->rootGroup);
             break;
 
+        case NoSorting: //manual and no grouping result both in non automatic grouping
+            break;
+
         default:
             kDebug() << "Invalid Strategy";
-            d->abstractSortingStrategy = 0;
     }
 
     d->sortingStrategy = sortOrder;
@@ -604,26 +659,26 @@ AbstractGroupingStrategy* GroupManager::taskGrouper() const
 
 void GroupManager::setGroupingStrategy(TaskGroupingStrategy strategy)
 {
-    if (d->changingGroupingStragegy ||
+    if (d->changingGroupingStrategy ||
         (d->abstractGroupingStrategy && d->abstractGroupingStrategy->type() == strategy)) {
         return;
     }
 
-    d->changingGroupingStragegy = true;
+    d->changingGroupingStrategy = true;
 
     //kDebug() << strategy << kBacktrace();
     if (d->onlyGroupWhenFull) {
-        disconnect(d->rootGroup, SIGNAL(itemAdded(AbstractItemPtr)), this, SLOT(checkIfFull()));
-        disconnect(d->rootGroup, SIGNAL(itemRemoved(AbstractItemPtr)), this, SLOT(checkIfFull()));
+        disconnect(d->rootGroup, SIGNAL(itemAdded(AbstractGroupableItem *)), this, SLOT(checkIfFull()));
+        disconnect(d->rootGroup, SIGNAL(itemRemoved(AbstractGroupableItem *)), this, SLOT(checkIfFull()));
     }
 
-    delete d->abstractGroupingStrategy;
-    d->abstractGroupingStrategy = 0;
+    if (d->abstractGroupingStrategy) {
+        disconnect(d->abstractGroupingStrategy, 0, this, 0);
+        d->abstractGroupingStrategy->destroy();
+        d->abstractGroupingStrategy = 0;
+    }
 
     switch (strategy) {
-        case NoGrouping:
-            d->abstractGroupingStrategy = 0;
-            break;
         case ManualGrouping:
             d->abstractGroupingStrategy = new ManualGroupingStrategy(this);
             break;
@@ -632,26 +687,32 @@ void GroupManager::setGroupingStrategy(TaskGroupingStrategy strategy)
             d->abstractGroupingStrategy = new ProgramGroupingStrategy(this);
             break;
 
+        case KustodianGrouping:
+            d->abstractGroupingStrategy = new KustodianGroupingStrategy(this);
+            break;
+
+        case NoGrouping:
+            break;
+
         default:
             kDebug() << "Strategy not implemented";
-            d->abstractGroupingStrategy = 0;
     }
 
     d->groupingStrategy = strategy;
 
-    if (d->groupingStrategy) {
+    if (d->abstractGroupingStrategy) {
         connect(d->abstractGroupingStrategy, SIGNAL(groupRemoved(TaskGroup*)),
                 this, SIGNAL(groupRemoved(TaskGroup*)));
     }
 
-    d->reloadTasks();
+    d->actuallyReloadTasks();
 
     if (d->onlyGroupWhenFull) {
-        connect(d->rootGroup, SIGNAL(itemAdded(AbstractItemPtr)), this, SLOT(checkIfFull()));
-        connect(d->rootGroup, SIGNAL(itemRemoved(AbstractItemPtr)), this, SLOT(checkIfFull()));
+        connect(d->rootGroup, SIGNAL(itemAdded(AbstractGroupableItem *)), this, SLOT(checkIfFull()));
+        connect(d->rootGroup, SIGNAL(itemRemoved(AbstractGroupableItem *)), this, SLOT(checkIfFull()));
     }
 
-    d->changingGroupingStragegy = false;
+    d->changingGroupingStrategy = false;
 }
 
 } // TaskManager namespace
