@@ -21,6 +21,7 @@
 #ifndef SYSTEMMODEL_H
 #define SYSTEMMODEL_H
 
+#include "core/kickoff_export.h"
 #include "core/kickoffproxymodel.h"
 
 namespace Kickoff
@@ -30,7 +31,7 @@ namespace Kickoff
  * Model which provides a tree of items for important system setup tools (eg. System Settings) ,
  * folders (eg. the user's home folder and the local network) and fixed and removable storage.
  */
-class SystemModel : public KickoffProxyModel
+class KICKOFF_EXPORT SystemModel : public KickoffProxyModel
 {
     Q_OBJECT
 
@@ -46,6 +47,7 @@ public:
     virtual int rowCount(const QModelIndex &parent = QModelIndex()) const;
     virtual int columnCount(const QModelIndex &parent = QModelIndex()) const;
     virtual QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
+    virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
 
 private Q_SLOTS:
     void startRefreshingUsageInfo();
