@@ -21,12 +21,13 @@
 
 //QT
 #include <QGraphicsSceneDragDropEvent>
-#include <QDesktopWidget>
 #include <QApplication>
 #include <QGraphicsLinearLayout>
 
 //KDE
 #include <KGlobalSettings>
+#include <KCModuleProxy>
+#include <KConfigDialog>
 #include <KDebug>
 #include <KLocale>
 #include <KIconLoader>
@@ -43,8 +44,9 @@
 #include <KIO/JobUiDelegate>
 
 //Plasma
-#include <plasma/widgets/icon.h>
-#include <plasma/containment.h>
+#include <Plasma/IconWidget>
+#include <Plasma/Containment>
+#include <Plasma/ToolTipManager>
 
 //Solid
 #include <solid/devicenotifier.h>
@@ -59,19 +61,20 @@
 Trash::Trash(QObject *parent, const QVariantList &args)
     : Plasma::Applet(parent, args),
       m_icon(0),
-      m_trashUrl(KUrl("trash:/")),
-      m_count(0)
+      m_dirLister(0),
+      emptyTrash(0),
+      m_count(0),
+      m_showText(false),
+      m_places(0),
+      m_proxy(0),
+      m_emptyProcess(0)
 {
+    setHasConfigurationInterface(true);
     setAspectRatioMode(Plasma::ConstrainedSquare);
-    setBackgroundHints(NoBackground);
-    m_icon = new Plasma::Icon(KIcon("user-trash"),QString(),this);
+
+    m_icon = new Plasma::IconWidget(KIcon("user-trash"),QString(),this);
     m_icon->setNumDisplayLines(2);
     m_icon->setDrawBackground(true);
-    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(this);
-    layout->setContentsMargins(0, 0, 0, 0);
-    layout->setSpacing(0);
-
-    layout->addItem(m_icon);
 
     resize(m_icon->sizeFromIconSize(IconSize(KIconLoader::Desktop)));
     createMenu();
@@ -79,13 +82,20 @@ Trash::Trash(QObject *parent, const QVariantList &args)
 
 Trash::~Trash()
 {
+    delete m_dirLister;
 }
 
 void Trash::init()
 {
-    m_places = new  KFilePlacesModel(this);
+    registerAsDragHandle(m_icon);
+
+    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(this);
+    layout->setContentsMargins(0, 0, 0, 0);
+    layout->setSpacing(0);
+    layout->addItem(m_icon);
 
     setAcceptDrops(true);
+    installSceneEventFilter(m_icon);
 
     m_dirLister = new KDirLister();
     connect( m_dirLister, SIGNAL( clear() ),
@@ -95,28 +105,30 @@ void Trash::init()
     connect( m_dirLister, SIGNAL( deleteItem( const KFileItem & ) ),
              this, SLOT( slotDeleteItem( const KFileItem & ) ) );
 
-    m_dirLister->openUrl(m_trashUrl);
-
-    registerAsDragHandle(m_icon);
-
-    //setMinimumSize(m_icon->sizeFromIconSize(IconSize(KIconLoader::Small)));
-    //FIXME PORT TO TOOLTIP MANAGER
-    //m_data.mainText = i18n("Trash");
+    m_dirLister->openUrl(KUrl("trash:/"));
 
     connect(m_icon, SIGNAL(activated()), this, SLOT(slotOpen()));
-    connect(&m_menu, SIGNAL(aboutToHide()),
-            m_icon, SLOT(setUnpressed()));
+}
+
+void Trash::createConfigurationInterface(KConfigDialog *parent)
+{
+    m_proxy = new KCModuleProxy("kcmtrash");
+
+    parent->addPage(m_proxy, i18n("Trash"), icon());
+    connect(parent, SIGNAL(okClicked()), this, SLOT(slotApplyConfig()));
+
+    m_proxy->load();
 }
 
 void Trash::createMenu()
 {
-    QAction* open = new QAction(SmallIcon("document-open"),i18n("&Open"), this);
+    QAction* open = new QAction(SmallIcon("document-open"), i18n("&Open"), this);
     actions.append(open);
     connect(open, SIGNAL(triggered(bool)), this , SLOT(slotOpen()));
 
-    emptyTrash = new QAction(SmallIcon("trash-empty"),i18n("&Empty Trashcan"), this);
+    emptyTrash = new QAction(SmallIcon("trash-empty"), i18n("&Empty Trashcan"), this);
     actions.append(emptyTrash);
-    connect(emptyTrash, SIGNAL(triggered(bool)), this , SLOT(slotEmpty()));
+    connect(emptyTrash, SIGNAL(triggered(bool)), this, SLOT(slotEmpty()));
 
     m_menu.addTitle(i18n("Trash"));
     m_menu.addAction(open);
@@ -126,6 +138,8 @@ void Trash::createMenu()
     QAction* menu = new QAction(SmallIcon("arrow-up-double"),i18n("&Menu"), this);
     connect(menu, SIGNAL(triggered(bool)), this , SLOT(popup()));
     m_icon->addIconAction(menu);
+
+    connect(&m_menu, SIGNAL(aboutToHide()), m_icon, SLOT(setUnpressed()));
 }
 
 void Trash::popup()
@@ -145,6 +159,7 @@ void Trash::constraintsEvent(Plasma::Constraints constraints)
     if (constraints & Plasma::FormFactorConstraint) {
         disconnect(m_icon, SIGNAL(activated()), this, SLOT(slotOpen()));
         disconnect(m_icon, SIGNAL(clicked()), this, SLOT(slotOpen()));
+
         if (formFactor() == Plasma::Planar ||
             formFactor() == Plasma::MediaCenter) {
 
@@ -167,85 +182,112 @@ void Trash::constraintsEvent(Plasma::Constraints constraints)
 
             setMinimumSize(m_icon->sizeFromIconSize(IconSize(KIconLoader::Small)));
         }
-        setIcon();
+        updateIcon();
     }
 }
 
 void Trash::slotOpen()
 {
     emit releaseVisualFocus();
-    KRun::runUrl(m_trashUrl, "inode/directory", 0);
+    KRun::runUrl(KUrl("trash:/"), "inode/directory", 0);
 }
 
 void Trash::slotEmpty()
 {
+    if (m_emptyProcess) {
+        return;
+    }
+
     emit releaseVisualFocus();
     const QString text(i18nc("@info", "Do you really want to empty the trash? All items will be deleted."));
+    //FIXME: should use KMessageBox::createKMessageBox so that it doesn't block the whole app!
     const bool del = KMessageBox::warningContinueCancel(&m_menu,
                                                         text,
                                                         QString(),
                                                         KGuiItem(i18nc("@action:button", "Empty Trash"),
                                                                   KIcon("user-trash"))
                                                         ) == KMessageBox::Continue;
+
     if (del) {
          // We can't use KonqOperations here. To avoid duplicating its code (small, though),
         // we can simply call ktrash.
         //KonqOperations::emptyTrash(&m_menu);
-        KProcess process;
-        process << KStandardDirs::findExe("ktrash") << "--empty";
-        process.execute();
-
+        emptyTrash->setEnabled(false);
+        emptyTrash->setText(i18n("Emptying Trashcan..."));
+        m_emptyProcess = new KProcess(this);
+        connect(m_emptyProcess, SIGNAL(finished(int,QProcess::ExitStatus)),
+                this, SLOT(emptyFinished(int,QProcess::ExitStatus)));
+        (*m_emptyProcess) << KStandardDirs::findExe("ktrash") << "--empty";
+        m_emptyProcess->start();
     }
 }
 
-void Trash::setIcon()
+void Trash::emptyFinished(int exitCode, QProcess::ExitStatus exitStatus)
 {
+    Q_UNUSED(exitCode)
+    Q_UNUSED(exitStatus)
+    //TODO: check the exit status and let the user know if it fails
+    delete m_emptyProcess;
+    m_emptyProcess = 0;
+    emptyTrash->setEnabled(true);
+    emptyTrash->setText(i18n("&Empty Trashcan"));
+}
+
+void Trash::updateIcon()
+{
+    Plasma::ToolTipContent data;
+    data.setMainText(i18n("Trash"));
+
     if (m_count > 0) {
-        m_icon->setIcon(KIcon("user-trash-full"));
-	//FIXME PORT TO TOOLTIP MANAGER
-        //m_data.subText = i18np("One item", "%1 items", m_count);
+        m_icon->setIcon("user-trash-full");
+
+        data.setSubText(i18np("One item", "%1 items", m_count));
         if (m_showText) {
             m_icon->setInfoText(i18np("One item", "%1 items", m_count));
         }
     } else {
-        m_icon->setIcon(KIcon("user-trash"));
-        //FIXME PORT TO TOOLTIP MANAGER
-	//m_data.subText = i18nc("The trash is empty. This is not an action, but a state", "Empty");
-        if (m_showText){
+        m_icon->setIcon("user-trash");
+
+        data.setSubText(i18nc("The trash is empty. This is not an action, but a state", "Empty"));
+        if (m_showText) {
             m_icon->setInfoText(i18nc("The trash is empty. This is not an action, but a state", "Empty"));
         }
     }
 
     m_icon->update();
-    
-    //FIXME TOOLTIP MANAGER
-    /*m_data.image = m_icon->icon().pixmap(IconSize(KIconLoader::Desktop));
+
+    data.setImage(m_icon->icon().pixmap(IconSize(KIconLoader::Desktop)));
 
     if (!m_showText) {
-        m_icon->setToolTip(m_data);
+        Plasma::ToolTipManager::self()->setContent(this, data);
     } else {
-        m_icon->setToolTip(Plasma::ToolTipData());
-    }*/
+        Plasma::ToolTipManager::self()->clearContent(this);
+    }
 
-    emptyTrash->setEnabled(m_count>0);
+    emptyTrash->setEnabled(m_count > 0);
 }
 
 void Trash::slotClear()
 {
     m_count = 0;
-    setIcon();
+    updateIcon();
 }
 
 void Trash::slotCompleted()
 {
     m_count = m_dirLister->items(KDirLister::AllItems).count();
-    setIcon();
+    updateIcon();
 }
 
 void Trash::slotDeleteItem(const KFileItem &)
 {
     m_count--;
-    setIcon();
+    updateIcon();
+}
+
+void Trash::slotApplyConfig()
+{
+    m_proxy->save();
 }
 
 QList<QAction*> Trash::contextualActions()
@@ -253,17 +295,54 @@ QList<QAction*> Trash::contextualActions()
     return actions;
 }
 
+void Trash::dragEnterEvent(QGraphicsSceneDragDropEvent *event)
+{
+    if (KUrl::List::canDecode(event->mimeData())) {
+        const KUrl::List urls = KUrl::List::fromMimeData(event->mimeData());
+        if (urls.count() == 1) {
+            if (!m_places) {
+                m_places = new KFilePlacesModel(this);
+            }
+
+            KUrl url = urls.at(0);
+
+            const Solid::Predicate predicate(Solid::DeviceInterface::StorageAccess, "filePath", url.path());
+
+            //query for mounted devices
+            const QList<Solid::Device> devList = Solid::Device::listFromQuery(predicate, QString());
+
+            //seek for an item in the places (e.g. Dolphin sidebar)
+            const QModelIndex index = m_places->closestItem(url);
+
+            if (!devList.isEmpty()) {
+                m_icon->setIcon("arrow-up-double");
+            } else if (m_places->bookmarkForIndex(index).url() == url) {
+                m_icon->setIcon("edit-delete");
+            }
+        }
+    }
+}
+
+void Trash::dragLeaveEvent(QGraphicsSceneDragDropEvent *event)
+{
+    updateIcon();
+}
+
 void Trash::dropEvent(QGraphicsSceneDragDropEvent *event)
 {
     if (KUrl::List::canDecode(event->mimeData())) {
         const KUrl::List urls = KUrl::List::fromMimeData(event->mimeData());
-        if (urls.count() > 0) {
+        if (!urls.isEmpty()) {
             event->accept();
 
             //some special operation was done instead of simply deleting a file
             bool specialOperation = false;
 
-             foreach (const KUrl& url, urls) {
+            if (!m_places) {
+                m_places = new KFilePlacesModel(this);
+            }
+
+            foreach (const KUrl& url, urls) {
                 const Solid::Predicate predicate(Solid::DeviceInterface::StorageAccess, "filePath", url.path());
 
                 //query for mounted devices
@@ -273,7 +352,7 @@ void Trash::dropEvent(QGraphicsSceneDragDropEvent *event)
                 //seek for an item in the places (e.g. Dolphin sidebar)
                 const QModelIndex index = m_places->closestItem(url);
 
-                if (devList.count() > 0) {
+                if (!devList.isEmpty()) {
                     //Assuming a mountpoint has a single device
                     Solid::Device device = devList.first();
 
@@ -291,16 +370,17 @@ void Trash::dropEvent(QGraphicsSceneDragDropEvent *event)
                 }
             }
 
+            if (specialOperation) {
+                updateIcon();
+
             //finally, try to trash a file
-             if (!specialOperation) {
+            }else{
                 KIO::Job* job = KIO::trash(urls);
                 job->ui()->setWindow(0);
                 job->ui()->setAutoErrorHandlingEnabled(true);
             }
-
         }
     }
-
 }
 
 #include "trash.moc"
