@@ -23,7 +23,7 @@
 #include <KDebug>
 #include <KLocale>
 
-#include "plasma/datacontainer.h"
+#include <Plasma/DataContainer>
 
 // The pattern here is:
 //	FreeBSD: param + mount
@@ -57,7 +57,7 @@ SolidDeviceEngine::SolidDeviceEngine(QObject* parent, const QVariantList& args)
     signalmanager = new DeviceSignalMapManager(this);
 
     listenForNewDevices();
-    temperature = new HddTemp();
+    temperature = new HddTemp(this);
     setMinimumPollingInterval(1000);
     connect(this, SIGNAL(sourceRemoved(const QString&)),
             this, SLOT(sourceRemoved(const QString&)));
@@ -65,8 +65,6 @@ SolidDeviceEngine::SolidDeviceEngine(QObject* parent, const QVariantList& args)
 
 SolidDeviceEngine::~SolidDeviceEngine()
 {
-    delete signalmanager;
-    delete temperature;
 }
 
 void SolidDeviceEngine::listenForNewDevices()
@@ -155,12 +153,17 @@ bool SolidDeviceEngine::populateDeviceData(const QString &name)
         devicetypes << I18N_NOOP("Storage Access");
         setData(name, I18N_NOOP("Accessible"), storageaccess->isAccessible());
         setData(name, I18N_NOOP("File Path"), storageaccess->filePath());
-        QVariant freeDiskVar;
-        qlonglong freeDisk = freeDiskSpace(storageaccess->filePath());
-        if ( freeDisk != -1 ) {
-            freeDiskVar.setValue( freeDisk );
+
+        if (storageaccess->isAccessible()) {
+            QVariant freeDiskVar;
+            qlonglong freeDisk = freeDiskSpace(storageaccess->filePath());
+            if ( freeDisk != -1 ) {
+                freeDiskVar.setValue( freeDisk );
+            }
+            if (!device.is<Solid::OpticalDisc>()) {
+                setData(name, I18N_NOOP("Free Space"), freeDiskVar );
+            }
         }
-        setData(name, I18N_NOOP("Free Space"), freeDiskVar );
 
         signalmanager->mapDevice(storageaccess, device.udi());
     }
@@ -265,10 +268,16 @@ bool SolidDeviceEngine::populateDeviceData(const QString &name)
         devicetypes << I18N_NOOP("Storage Volume");
 
         QStringList usagetypes;
-        usagetypes << I18N_NOOP("File System") << I18N_NOOP("Partition Table") << I18N_NOOP("Raid") << I18N_NOOP("Other") << I18N_NOOP("Unused");
+        usagetypes << i18n("Other") << i18n("Unused") << i18n("File System")
+                   << i18n("Partition Table") << i18n("Raid") << i18n("Encrypted");
+
+        if (usagetypes.count() > storagevolume->usage()) {
+            setData(name, I18N_NOOP("Usage"), usagetypes.at((int)storagevolume->usage()));
+        } else {
+            setData(name, I18N_NOOP("Usage"), i18n("Unknown"));
+        }
 
         setData(name, I18N_NOOP("Ignored"), storagevolume->isIgnored());
-        setData(name, I18N_NOOP("Usage"), usagetypes.at((int)storagevolume->usage()));
         setData(name, I18N_NOOP("File System Type"), storagevolume->fsType());
         setData(name, I18N_NOOP("Label"), storagevolume->label());
         setData(name, I18N_NOOP("Uuid"), storagevolume->uuid());
@@ -513,7 +522,7 @@ qlonglong SolidDeviceEngine::freeDiskSpace(const QString &mountPoint)
     if (statvfs(path,&fs_obj) < 0) {
         return -1;
     } else {
-        return (qlonglong)fs_obj.f_bfree*(qlonglong)fs_obj.f_frsize;
+        return (qlonglong)fs_obj.f_bavail*(qlonglong)fs_obj.f_frsize;
     }
 #elif defined(HAVE_STATFS) && !defined(USE_SOLARIS)
     struct statfs fs_obj;
@@ -534,8 +543,10 @@ qlonglong SolidDeviceEngine::freeDiskSpace(const QString &mountPoint)
 bool SolidDeviceEngine::updateFreeSpace(const QString &udi)
 {
     Solid::Device device = devicemap.value(udi);
-    if (!device.is<Solid::StorageAccess>()) {
+    if (!device.is<Solid::StorageAccess>() || device.is<Solid::OpticalDisc>()) {
         return false;
+    } else if (!device.as<Solid::StorageAccess>()->isAccessible()) {
+        removeData(udi, I18N_NOOP("Free Space"));
     }
 
     Solid::StorageAccess *storageaccess = device.as<Solid::StorageAccess>();
